/*
** Module   : ErrorConditionException.java
** Abstract : represents a Progress ERROR condition
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description------------------------------
** 001 GES 20050811   @22111 Created initial version which represents a
**                           Progress ERROR condition.
** 002 GES 20060505   @25986 Added a flag to differentiate the case where
**                           the cause of this condition was the END-ERROR
**                           key.
** 003 GES 20060623   @27561 Support storing a key code as a cause.
** 004 SVL 20090908   @43848 Added progressErrorCode field. 
** 005 CA  20130612          Set the cause of this condition a NumberedException referring the
**                           error code and associated error message.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.NumberedException;

/**
 * Represents a Progress ERROR condition.
 */
public class ErrorConditionException
extends ConditionException
{
   /** <code>true</code> if the cause of this error was the END-ERROR key. */
   private boolean end_error = false;

   /** Error code used in Progress 4GL. */
   private int progressErrorCode = -1;
   
   /**
    * Constructor which accepts an error message.
    *
    * @param    message
    *           Error message.
    */
   public ErrorConditionException(String message)
   {
      super(message);
   }

   /**
    * Constructor which accepts an error message and an error code.
    *
    * @param    progressErrorCode
    *           Error code (which is used in Progress 4GL).
    * @param    message
    *           Error message.
    */
   public ErrorConditionException(int progressErrorCode, String message)
   {
      super(message);
      this.progressErrorCode = progressErrorCode;
      initCause(new NumberedException(message, progressErrorCode));
   }
    
   /**
    * Constructor which accepts an error message, error code and root cause.
    *
    * @param    progressErrorCode
    *           Error code (which is used in Progress 4GL).
    * @param    message
    *           Error message.
    * @param    cause
    *           A numbered exception identifying the raised condition.
    */
   public ErrorConditionException(int progressErrorCode, String message, NumberedException cause)
   {
      super(message, cause);
      this.progressErrorCode = progressErrorCode;
   }

  /**
    * Constructor which accepts an error message and the key code for the key
    * press that caused this condition.
    *
    * @param   message
    *          Error message.
    * @param   keycode
    *          The key code of the key that caused this condition to be
    *          raised.
    */
   public ErrorConditionException(String message, int keycode)
   {
      super(message, keycode);
   }
    
   /**
    * Constructor which accepts an error message and a root cause. This
    * variant is used when the error condition is detected as the result of
    * catching another exception.
    *
    * @param    message
    *           Error message.
    * @param    rootCause
    *           Exception which has been caught in the course of processing.
    */
   public ErrorConditionException(String message, Throwable rootCause)
   {
      super(message, rootCause);
   }
    
   /**
    * Constructor which accepts and wrappers a root cause. This variant is
    * used when the error condition is detected as the result of catching
    * another exception, and the message or context of the root cause is
    * sufficient to describe the problem.
    *
    * @param    rootCause
    *           Exception which has been caught in the course of processing.
    */
   public ErrorConditionException(Throwable rootCause)
   {
      super(rootCause);
   }
    
   /**
    * Constructor which accepts an error message.
    *
    * @param    message
    *           Error message.
    * @param    end_error
    *           <code>true</code> if the cause of this condition was the
    *           END-ERROR key.
    */
   public ErrorConditionException(String message, boolean end_error)
   {
      super(message);
      this.end_error = end_error;
   }
   
   /**
    * Constructor which accepts an error message and the key code for the key
    * press that caused this condition.
    *
    * @param    message
    *           Error message.
    * @param    keycode
    *           The key code of the key that caused this condition to be
    *           raised.
    * @param    end_error
    *           <code>true</code> if the cause of this condition was the
    *           END-ERROR key.
    */
   public ErrorConditionException(String  message, 
                                  int     keycode,
                                  boolean end_error)
   {
      super(message, keycode);
      this.end_error = end_error;
   }

  /**
    * Detects if the cause of this condition was the END-ERROR key.
    *
    * @return   <code>true</code> if the cause of this condition was the
    *           END-ERROR key.
    */
   public boolean isEndError()
   {
      return end_error;
   }
   
   /**
    * Sets the END-ERROR flag for the cause of this condition.
    *
    * @param    end_error
    *           <code>true</code> if the cause of this condition was the
    *           END-ERROR key.
    */
   public void setEndError(boolean end_error)
   {
      this.end_error = end_error;
   }

   /**
    * Returns the code (which is used in Progress 4GL) of the error that was
    * the cause of this condition.
    *
    * @return   The code of the error that caused this condition or -1 if it
    *           wasn't specified.
    */
   public int getProgressErrorCode()
   {
      return progressErrorCode;
   }
}
